// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scenes/PixelVNScene.hpp"
#include "scenes/Scene_2.hpp"
#include "prefabs/characters/DeathPrefab.hpp"
#include "scene/components/audio/AudioListener.hpp"
#include "scene/components/audio/AudioSource.hpp"
#include "visualnovel/events/characters/VisualNovelFadeCharacterEvent.hpp"
#include "visualnovel/events/timing/VisualNovelBatchEvent.hpp"

namespace Dawn {
  class Scene_1 : public PixelVNScene {
    protected:
      DeathPrefab *death;
      DeathPrefab *death2;

      void vnStage() override {
        PixelVNScene::vnStage();
        
        this->death = DeathPrefab::create(this);
        this->death2 = DeathPrefab::create(this);
        this->death2->transform.setLocalPosition(glm::vec3(100, 0, 0));
      }

      void onSceneEnded() {
        auto scene = new SubSceneRendererScene<Scene_2>(this->game);
        game->assetManager.queueSwap(
          scene->getRequiredAssets(), this->getRequiredAssets()
        );
        game->assetManager.syncLoad();
        scene->stage();
        this->game->sceneCutover(scene);
      }

    public:
      Scene_1(DawnGame *game) : PixelVNScene(game) {
        
      }

      std::vector<Asset*> getRequiredAssets() override {
        auto man = &this->game->assetManager;
        std::vector<Asset*> assets = PixelVNScene::getRequiredAssets();
        vectorAppend(&assets, DeathPrefab::getRequiredAssets(man));
        assets.push_back(man->get<AudioAsset>("audio_test"));
        return assets;
      }

      IVisualNovelEvent * getVNEvent() override {
        auto start = new VisualNovelPauseEvent(vnManager, 0.1f);
        start
          ->then(new VisualNovelBatchEvent(
            vnManager,
            std::vector<IVisualNovelEvent*>{
              new VisualNovelFadeCharacterEvent(
                vnManager,
                this->death->vnCharacter,
                true,
                &easeLinear,
                2.0f
              ),
              new VisualNovelFadeCharacterEvent(
                vnManager,
                this->death2->vnCharacter,
                false,
                &easeLinear,
                2.0f
              )
            }
          ))
          ->then(new VisualNovelTextboxEvent(vnManager, nullptr, "scene.1.1"))
          // ->then(new VisualNovelCallbackEvent<Scene_1>(vnManager, this, &Scene_1::onSceneEnded))
        ;
        return start;
      }
  };
}